from os import makedirs
from os.path import exists, join
from shutil import rmtree
from subprocess import check_call

from setuptools import Command


class PyzCommand(Command):
    description = f"""
            Build runtime pyz executable using shiv.
            """
    DEFAULT_DIR = "./dist"
    DEFAULT_EP = "hashkernel.cli:main"

    user_options = [
        ("dir=", None, f"where to create .pyz file, default is: {DEFAULT_DIR}"),
        ("name=", None, f"name of .pyz file, if omitted defaults to package name"),
        (
            "script=",
            None,
            f"entry point used in .pyz file created, default is {DEFAULT_EP}",
        ),
    ]

    def initialize_options(self):
        self.dir = self.DEFAULT_DIR
        self.script = self.DEFAULT_EP
        self.name = f"{self.distribution.get_name()}.pyz"

    def target_dir(self):
        return join(self.dir, "site_packages_for_pyz")

    def finalize_options(self):
        pass

    def run(self):
        """
        Execute command. Create runenv and install dependencies
        """
        if not exists("setup.py"):
            raise ValueError(f"Has to be run from root of the project")
        rmtree(self.target_dir(), ignore_errors=True)
        makedirs(self.target_dir(), exist_ok=True)
        check_call(["pip", "install", ".", "--target", self.target_dir()])
        makedirs(self.dir, exist_ok=True)
        self.run_shiv(self.script)

    def run_shiv(self, ep_method):
        pyz_name = join(self.dir, self.name)
        check_call(
            [
                "shiv",
                "--site-packages",
                self.target_dir(),
                "-o",
                pyz_name,
                "-e",
                ep_method,
            ]
        )
        print(f"Created: {pyz_name} with entry point: {ep_method}")



import locale
from os.path import exists, sep
from subprocess import check_call, check_output
from venv import EnvBuilder

from setuptools import Command


class RunEnv(Command):
    description = f"""
            Build runtime venv for given project 
            and install it there.
            """
    DEFAULT_DIR = f".{sep}.runenv"

    user_options = [
        ("dir=", None, f"where to create runenv, default is: {DEFAULT_DIR}")
    ]

    def initialize_options(self):
        self.dir = self.DEFAULT_DIR

    def finalize_options(self):
        pass

    def run(self):
        """
        Execute command. Create runenv and install dependencies
        """
        if not exists("setup.py"):
            raise ValueError(f"Has to be run from root of the project")
        builder = EnvBuilder(
            system_site_packages=False, clear=False, symlinks=False, with_pip=False
        )
        builder.create(self.dir)
        site_packages = (
            check_output(
                [f"{self.dir}/bin/python", "-Ic", "import sys; print(sys.path[-1])"]
            )
            .decode(locale.getpreferredencoding(False))
            .strip()
        )

        check_call(["pip", "install", ".", "--target", site_packages])



import os
import shutil
import sys
import time
from logging import getLogger

pyenv = f"py{sys.version.split()[0]}"


SEC_DELAYS = [0.0, *(0.05 * (1.4 ** i) for i in range(3))]


def negate(fn):
    def nope(*args, **kwargs):
        return not fn(*args, **kwargs)

    return nope


def repeat_action_with_delays(test_fn, action_fn, *args):
    for delay in SEC_DELAYS:
        if test_fn(*args):
            action_fn(*args)
            time.sleep(delay)
        else:
            break


def ensure_dir(d):
    repeat_action_with_delays(negate(os.path.isdir), os.makedirs, d)


def ensure_no_dir(dir):
    repeat_action_with_delays(os.path.isdir, shutil.rmtree, dir)


class LogTestOut:
    """
    Get logger and initalize `./test-out/py3.x.x/test_name` directory.

    >>> log, out = LogTestOut.get(__name__)
    >>> log is not None
    True
    >>> isinstance(out, LogTestOut)
    True
    >>> dir = out.child_dir("home") # ensure child directory
    >>> os.path.isdir(dir)
    True
    >>> log, out = LogTestOut.get(__name__) # reinitialise and force cleanup
    >>> os.path.isdir(dir)
    False
    """

    def __init__(self, name, root):
        self.log = getLogger(name)
        if root is None:
            root = os.path.abspath("test-out")
        self.dir = os.path.join(root, pyenv, name)
        ensure_no_dir(self.dir)
        self.child_dirs = {}

    def child_dir(self, dir_name):
        if dir_name not in self.child_dirs:
            child = os.path.join(self.dir, dir_name)
            ensure_dir(child)
            self.child_dirs[dir_name] = child
        return self.child_dirs[dir_name]

    @classmethod
    def get(cls, name, root=None):
        out = cls(name, root)
        return out.log, out



import os.path
from distutils import dir_util
from distutils.command.clean import clean as CleanCommand
from pathlib import Path

REMOVE_DIRS = {
    "dist",
    "build",
    ".mypy_cache",
    ".pytest_cache",
    "htmlcov",
    "*.egg-info",
    "**/__pycache__",
    "**/.runenv",
}

RUN_UTILS = ["isort .", "black ."]


class TidyCommand(CleanCommand):
    def initialize_options(self):
        CleanCommand.initialize_options(self)

    def finalize_options(self):
        CleanCommand.finalize_options(self)
        self.all = True

    def run(self):
        CleanCommand.run(self)
        curdir = Path(".")
        dir_names = set(
            str(f) for p in REMOVE_DIRS for f in curdir.glob(p) if f.is_dir()
        )

        for dir_name in dir_names:
            if os.path.exists(dir_name):
                dir_util.remove_tree(dir_name, dry_run=self.dry_run)
            else:
                self.announce(f"skipping {dir_name} since it does not exist")

        for cmd in RUN_UTILS:
            if 0 != os.system(cmd):
                self.announce(f"unsuccessful cmd: {cmd}")



def build_assert_text(ok_, eq_):
    def body(src, expect, save_words=None):
        src_it = iter(src.split())
        expect_it = iter(expect.split())
        look_until_match = False
        s1, s2 = None, None
        while True:
            try:
                s1 = next(src_it)
            except StopIteration:
                try:
                    s2 = next(expect_it)
                except StopIteration:
                    break
                print(src)
                ok_(False, "expecting longer text %r %r" % (s1, s2))
            if look_until_match:
                if s2 == s1:
                    look_until_match = False
                continue
            try:
                s2 = next(expect_it)
            except StopIteration:
                print(src)
                ok_(False, "expecting shorter text")
            if s2 == "....":
                look_until_match = True
                s2 = None
                try:
                    s2 = next(expect_it)
                except StopIteration:
                    pass
            elif s2 == "...":
                if save_words is not None:
                    save_words.append(s1)
            else:
                if s1 != s2:
                    print(src)
                    eq_(s1, s2)

    return body



from hs_build_tools.assert_text_template import build_assert_text


def ok_(b, msg=None):
    if msg is None:
        assert b
    else:
        assert b, msg


def eq_(a, b, msg=None):
    if msg is None:
        assert a == b
    else:
        assert a == b, msg


assert_text = build_assert_text(ok_, eq_)


def doctest_for_assert_text():
    """
    matches texts ignoring spaces.

    Some fragment of the `src` could be ignored if it is maked
    inside `expect` with placeholders:

    ... - ignore word
    .... - ignore any number of words until next word match

    TODO: it works but it is inconsistent with ellipsis syntax in
    doctest. I guess you could see bit of irony bellow.

    >>> assert_text('abc   xyz', ' abc xyz ')
    >>> assert_text('abc asdf  xyz', ' abc ... xyz ')
    >>> assert_text('abc asdf iklmn xyz', ' abc ... xyz ')
    Traceback (most recent call last):
    ...
    AssertionError
    >>> assert_text('abc asdf iklmn xyz', ' abc .... xyz ')
    >>> save_vars = []
    >>> pattern = ' abc ... rrr ... xyz '
    >>> assert_text('abc asdf rrr qqq xyz', pattern, save_vars)
    >>> save_vars
    ['asdf', 'qqq']

    """
    ...



from doctest import testmod

from nose.tools import eq_, ok_

from hs_build_tools.assert_text_template import build_assert_text


def doctest_it(m):
    results = testmod(m)
    ok_(results.attempted > 0, "There is not doctests in module")
    eq_(results.failed, 0)


assert_text = build_assert_text(ok_, eq_)


def doctest_for_assert_text():
    """
    matches texts ignoring spaces.

    Some fragment of the `src` could be ignored if it is maked
    inside `expect` with placeholders:

    ... - ignore word
    .... - ignore any number of words until next word match

    TODO: it works but it is inconsistent with ellipsis syntax in
    doctest. I guess you could see bit of irony bellow.

    >>> assert_text('abc   xyz', ' abc xyz ')
    >>> assert_text('abc asdf  xyz', ' abc ... xyz ')
    >>> assert_text('abc asdf iklmn xyz', ' abc ... xyz ')
    Traceback (most recent call last):
    ...
    AssertionError: 'iklmn' != 'xyz'
    >>> assert_text('abc asdf iklmn xyz', ' abc .... xyz ')
    >>> save_vars = []
    >>> pattern = ' abc ... rrr ... xyz '
    >>> assert_text('abc asdf rrr qqq xyz', pattern, save_vars)
    >>> save_vars
    ['asdf', 'qqq']

    """
    ...



import platform

from setuptools import find_packages, setup

from hs_build_tools.pyz import PyzCommand
from hs_build_tools.runenv import RunEnv
from hs_build_tools.tidy import TidyCommand

cmdclass_dict = {"runenv": RunEnv, "tidy": TidyCommand, "pyz": PyzCommand}

install_requires = ["shiv"]

dev_requires = [
    "sniffer",
    "coverage",
    "mypy",
    "twine",
    "wheel",
    "nose",
    "pytest",
    "isort",
    "black",
]

makes_sniffer_scan_faster = {
    "Linux": "pyinotify",
    "Windows": "pywin32",
    "Darwin": "MacFSEvents",
}

if platform.system() in makes_sniffer_scan_faster:
    dev_requires.append(makes_sniffer_scan_faster[platform.system()])


def read_file(f):
    with open(f, "r") as fh:
        return fh.read()


long_description = read_file("README.md")

try:
    from hs_build_tools.release import get_version_and_add_release_cmd

    version = get_version_and_add_release_cmd("version.txt", cmdclass_dict)
except ModuleNotFoundError:
    version = read_file("version.txt").strip()


setup(
    name="hs_build_tools",
    version=str(version),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
    ],
    description="hashstore build tools",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/walnutgeek/hashstore",
    author="Walnut Geek",
    author_email="wg@walnutgeek.com",
    license="Apache 2.0",
    packages=find_packages(exclude=("tests",)),
    package_data={},
    cmdclass=cmdclass_dict,
    entry_points={
        "distutils.commands": [
            "runenv = hs_build_tools.runenv:RunEnv",
            "tidy = hs_build_tools.tidy:TidyCommand",
            "pyz = hs_build_tools.pyz:PyzCommand",
        ]
    },
    install_requires=install_requires,
    extras_require={"dev": dev_requires},
    zip_safe=True,
)



import locale
from datetime import date, timedelta
from subprocess import CalledProcessError, check_call, check_output

from setuptools import Command


def next_month(d):
    """
    >>> next_month(date(2019,6,19))
    datetime.date(2019, 7, 1)
    >>> next_month(date(2019,12,19))
    datetime.date(2020, 1, 1)
    """
    cur_month = d.month
    while cur_month == d.month:
        d += timedelta(days=1)
    return d


class Version:
    """

    >>> Version((2019, 7, 1)).next_minor(date(2019,6,19))
    Version((2019, 7, 2))
    >>> Version((2019, 7, 1)).next_major(date(2019,6,19))
    Traceback (most recent call last):
    ...
    ValueError: cannot calc major version from: (2019, 7, 1) 2019-07-01
    >>> Version((2019, 7, 1)).next_major(date(2019,7,1))
    Version((2019, 8))

    """

    def __init__(self, s):
        self.nums = s if isinstance(s, tuple) else tuple(map(int, s.split(".")))
        if len(self.nums) not in (2, 3):
            raise ValueError(f"version has to have 2 or 3 parts: {s}")

    def __str__(self):
        return ".".join(map(str, self.nums))

    def __repr__(self):
        return f"Version({self.nums})"

    def next_major(self, now=date.today()):
        _nums = (now.year, now.month)
        if _nums > self.nums:
            return Version(_nums)
        else:
            now = next_month(now)
            _nums = (now.year, now.month)
            if _nums > self.nums:
                return Version(_nums)
            raise ValueError(f"cannot calc major version from: {self.nums} {now}")

    def next_minor(self, now=date.today()):
        last_num = 1
        if self.is_minor():
            last_num = self.nums[2]
        date_t = max(tuple(self.nums[:2]), (now.year, now.month))
        _nums = (*date_t, last_num)
        if self.nums == _nums:
            return Version((*date_t, last_num + 1))
        else:
            _nums = (now.year, now.month, 1)
            if _nums > self.nums:
                return Version(_nums)
            now = next_month(now)
            _nums = (now.year, now.month, 1)
            if _nums > self.nums:
                return Version(_nums)
            raise ValueError(f"cannot calc version from: {self.nums} {now}")

    def is_minor(self):
        return len(self.nums) == 3

    def type(self):
        return "minor" if self.is_minor() else "major"


def get_version_and_add_release_cmd(version_file, cmdclass_dict):
    version = Version(open(version_file).read().strip())

    class ReleaseCommand(Command):
        description = f"""
            Check release:
                return success errorCode if {version_file} match 
                current tag on branch. 
    
            Trigger release:
                change {version_file}, tag and push changes to git.
    
            """

        user_options = [
            ("azure", None, "publish azure vars"),
            ("minor", None, "trigger minor release "),
            ("major", None, "trigger major release "),
        ]

        def initialize_options(self):
            self.azure = self.minor = self.major = False

        def finalize_options(self):
            pass

        def azure_var(self, key, value):
            if self.azure:
                print(
                    f"##vso[task.setvariable " f"variable={key};isOutput=true]{value}"
                )

        def run(self):
            """Run command."""
            new_ver = None
            if self.major:
                new_ver = version.next_major()
            elif self.minor:
                new_ver = version.next_minor()
            else:
                try:
                    tag = (
                        check_output(["git", "describe", "--tags", "--exact-match"])
                        .decode(locale.getpreferredencoding(False))
                        .strip()
                    )
                    version_ = f"v{version}"
                    print(f"version.txt={version_!r} git={tag!r}")
                    match = tag == version_
                except CalledProcessError:
                    print(f"no tag found")
                    match = False
                    self.azure_var("type", "none")
                if match:
                    print(f"{version.type()} release. Git tag matched.")
                    self.azure_var("type", version.type())
                raise SystemExit(0)
            open(version_file, "wt").write(str(new_ver))
            print(f"New version: {new_ver}")
            tag = f"v{new_ver}"
            msg = ["-m", tag]
            check_call(["git", "add", version_file])
            check_call(["git", "commit", *msg])
            check_call(["git", "tag", "-a", tag, *msg])
            # check_call(f'git push origin --tags'.split())
            check_call("git push --tags origin HEAD".split())
            check_call("git push -u origin".split())

    cmdclass_dict["release"] = ReleaseCommand
    return version
